var gMap = (function(){
	
	/**
	 * 
	 * */
	var geocoder,
	
	/**
	 * 
	 * */
		map,
		
	/**
	 * 
	 * */
		initialLocation,
		
	/**
	 * marker object
	 * */
		marker = null,
		
	/**
	 * elevator object
	 * */
		elevator,
		
	/**
	 * object InfoWindow for displaying found location
	 * */
		locationInfo,
		
		browserSupportFlag,
		
		devMode = false;
	
	/**
	 * 
	 * */
	function initMap() {
		
		geocoder = new google.maps.Geocoder();
		locationInfo = new google.maps.InfoWindow();
		initialLocation = new google.maps.LatLng(52.066667,19.483333);
		elevator = new google.maps.ElevationService();

		/** set center Poland point */
		map = new google.maps.Map(document.getElementById("map"), {
			zoom: 5,
			mapTypeId: google.maps.MapTypeId.ROADMAP,
			center: initialLocation,
			scrollwheel: false,
			panControl: true,
			zoomControl: true,
			disableDoubleClickZoom: true,
			zoomControlOptions: {
		      style: google.maps.ZoomControlStyle.LARGE
		    },
			mapTypeControl: true,
		  	scaleControl: true,
		  	streetViewControl: false
		});

		google.maps.event.addListener(map, 'click', function( event ) {
			var latLng = event.latLng; 
			handleManualGeo(latLng);
		});
		marker = new google.maps.Marker({map:map});  
		//geolocateUser();
		bindEvents();
		
	}
	
	function geolocateUser(){
		// Try W3C Geolocation (Preferred)
		if(navigator.geolocation) {
			browserSupportFlag = true;
		    navigator.geolocation.getCurrentPosition(function(position) {
		    	initialLocation = new google.maps.LatLng(position.coords.latitude,position.coords.longitude);
		    	map.setCenter(initialLocation);
		    }, function() {
		    	handleNoGeolocation(browserSupportFlag);
		    });
		}
		else {
			browserSupportFlag = false;
			handleNoGeolocation(browserSupportFlag);
		}
	}

	function handleNoGeolocation(errorFlag) {
	    if (errorFlag == true) {
	    	debug("Geolocation service failed.");
	    } else {
	    	debug("Your browser doesn't support geolocation. We've placed you on center point of Poland.");
	    }
	    map.setCenter(initialLocation);
	  }

	function debug( msg ){
	
		if( devMode )opera.postError( msg );
	}
	
	function trimString(str){
		return str.replace(/^\s+|\s+$/g, '');
	}
	function bindEvents(){
		
		$( '#btnGeocode' ).click(function(){
			codeAddress();
		});
		
		$( '#searchAddress' ).keypress(function(e){
			if( e.which == '13' ) {
				codeAddress();
			}
		});
		
		$( '#btnClose' ).click(function(){
			$( '#overlay' ).css( {top:'100%'} );
		});
	}
	/**
	 * 
	 * */
	function handleManualGeo( latLng ){
		$('#geoAddresses').empty();
	    $( '#loader' ).removeClass( 'active' );
	    $( '#overlay' ).css( {top:'100%'} );
	    $( '#outputPanel input' ).val('');
	    locationInfo.close();

	    marker.setPosition(latLng);
		
		if( map.getZoom() < 13 ) map.setZoom(12); 
		map.setCenter( latLng );
		var location = {name:'',position: latLng}; 
		viewOutput( location );
		//viewLocationWindow({name:'',position: latLng});
	}
	/**
	 * geocode an address and set map center at the returned latitude and longitude values
	 * */
	function codeAddress() {
	    var address = document.getElementById( "searchAddress" ).value;
	    address = trimString(address);
	    $('#geoAddresses').empty();
	    $( '#overlay' ).css( {top:'100%'} );
	    $( '#outputPanel input' ).val('');
	    
	    locationInfo.close();
	    $( '#loader' ).addClass( 'active' );
	    geocoder.geocode( { 'address': address }, function( results, status ) {
	    	$( '#loader' ).removeClass( 'active' );
	    	if (status == google.maps.GeocoderStatus.OK ) {
	    		if( results.length == 1 ) {
	    			var location = formatGeocoderResult(results[0]);
	    			zoomLocation(location);
	    		} else if( results.length > 1 ){
	    			var locations = [];
	    			jQuery.each( results, function(i,result) {
	    				locations.push( formatGeocoderResult( result ));
	    			});
	    			viewLocationSelect(locations);
	    		}
	    	} else {
	    		viewLocationSelect([]);
	    	}
    	});
	}

	/**
	 * function getElevation
	 * */
	function getElevation(latLng) {

		// Create a LocationElevationRequest object using the array's one value
		var positionalRequest = {
			'locations': [latLng]
		}

		// Initiate the location request
		elevator.getElevationForLocations( positionalRequest, function(results, status) {
			if (status == google.maps.ElevationStatus.OK) {
				if (results[0]) {
					
					var oldContent = locationInfo.getContent();
					// Open an info window indicating the elevation at the clicked position
					locationInfo.setContent(  (oldContent ? oldContent : '') + "<p>The elevation at this point is " + results[0].elevation + " meters.</p>");
					locationInfo.setPosition(latLng);
					locationInfo.open(map);
				} else {
		        //alert("No results found");
				}
			} else {
		      //alert("Elevation service failed due to: " + status);
			}
		});
	}
	
	/**
	 * 
	 * */
	function formatGeocoderResult( result ){
		
		var ret = {
			raw_name: result.formatted_address,
			name: result.formatted_address,
			position: result.geometry.location,
			locality: '',
			administrativeRegion: '',
			country: '',
			raw_tags: []
		};
		
		var addr_components = result.address_components;
		for( var index=0; index<addr_components.length; index++ ) {
			var component = addr_components[index];
			if( jQuery.inArray( 'administrative_area_level_1', component.types ) > -1 ) {
				ret.administrativeRegion = component;
				ret.name += ' (' + component.short_name + ')';
			}
			if( jQuery.inArray( 'country', component.types ) > -1 ) {
				ret.country = component;
			}
			if( jQuery.inArray( 'administrative_area_level_2', component.types ) > -1 ) {
				ret.administrativeRegion2 = component;
			}
			if( jQuery.inArray( 'administrative_area_level_3', component.types ) > -1 ) {
				ret.administrativeRegion3 = component;
			}
			if( jQuery.inArray( 'locality', component.types ) > -1 ) {
				ret.locality = component;
			}
			if( component.long_name ){
				ret.raw_tags.push( component.long_name );
			}
		}
		return ret;
	}
	/**
	 * 
	 * */
	function viewLocationSelect(locations){
		var outDiv = $('#geoAddresses');
		outDiv.empty();
		
		if(locations.length){
			var txt = '<p>Few matching addresses were found, click on the location to see it on the map:</p>';
			 	txt += '<ul>';
		
			jQuery.each( locations, function(i,location){
				txt += '<li class="location" id="location_'+ i +'">' + location.name + '</li>';
			});
			txt += '</ul>';
			outDiv.html(txt);
			
			$('.location').click(function(){
				$( '#overlay' ).css( {top:'100%'} );
				var ind = (this.id).split('_')[1];
				zoomLocation(locations[ind]);
				
			});
		} else {
			var txt = '<p>Location not found.</p>';
			outDiv.html(txt);
		}
		//outDiv.show();
		$( '#overlay' ).animate({top: '3em', display: 'block'}, 500);
	}

	/**
	 * 
	 * */
	function zoomLocation(location){
		debug(  location.name );
		map.setCenter( location.position );
		if( map.getZoom() < 13 ) map.setZoom(12); 
		
		//locationInfo.setOptions({pixelOffset: new google.maps.Size( -3,-20)} );
		viewOutput( location );
		marker.setPosition(location.position);
		//viewLocationWindow(location);
	}
	
	/**
	 * 
	 * */
	function viewLocationWindow(location){
		
		var positionalRequest = {
				'locations': [location.position]
		}

		// Initiate the location request
		elevator.getElevationForLocations( positionalRequest, function(results, status) {
			if (status == google.maps.ElevationStatus.OK) {
				if (results[0]) {
					// Open an info window indicating the elevation at the clicked position
					locationInfo.setContent(  '<h5>'+location.name+'</h5>' + "<p>The elevation at this point is " + results[0].elevation + " meters.</p>");
					locationInfo.setPosition(location.position);
					locationInfo.open(map);
				} else {
		        //alert("No results found");
				}
			} else {
				if( location.name != '' ) {
					locationInfo.setContent('<h5>'+location.name+'</h5>');
					locationInfo.setPosition(location.position);
					locationInfo.open(map);
				}
			}
			
		});
	}
	
	/**
	 * @param {Object} location
	 * @return {Object} data with lat,lng values in degrees
	 * */
	function formatLatLng( location ){
		
		var lat = Math.abs( location.position.lat() ),
			latDeg = Math.floor(lat),
			latMin = Math.floor((lat-latDeg)*60);
		
		var lon = Math.abs( location.position.lng() ),
			lonDeg = Math.floor(lon),
			lonMin = Math.floor((lon-lonDeg)*60);

	    
	    var ret = {
	    	latitude: {
	    		raw: location.position.lat(),
	    		degrees: latDeg,
	    		minutes: latMin,
	    		seconds: (Math.round(((((lat - latDeg) - (latMin/60)) * 60 * 60) * 100) / 100) ),
	    		direction: ( location.position.lat() < 0 ? "S" : "N" )
	    	},
	    	longitude: {
	    		raw: location.position.lng(),
	    		degrees: lonDeg,
	    		minutes: lonMin,
	    		seconds: (Math.round(((((lon - lonDeg) - (lonMin / 60 )) * 60 * 60) * 100 ) / 100)),
	    		direction: ( location.position.lng() < 0 ? "W" : "E" )
	    	}
	    };
	    return ret;
	}

	function viewOutput( location ){
		
		var latLng  = formatLatLng( location );
		//var latTime = latLng.latitude.degrees + '&#176;&nbsp;' +  latLng.latitude.minutes + '&#39;&nbsp;' +  latLng.latitude.seconds + '&#39;&#39;&nbsp;' + latLng.latitude.direction ;
		//var lngTime = latLng.longitude.degrees + '&#176;&nbsp;' +  latLng.longitude.minutes + '&#39;&nbsp;' +  latLng.longitude.seconds + '&#39;&#39;&nbsp;' + latLng.longitude.direction;
		//console.log( JSON.stringify( latLng ));
		$('#lat').val( latLng.latitude.raw );
		$('#lng').val( latLng.longitude.raw );

		$('#lat_degree').val( latLng.latitude.degrees );
		$('#lng_degree').val( latLng.longitude.degrees );
		$('#lat_minute').val( latLng.latitude.minutes );
		$('#lng_minute').val( latLng.longitude.minutes );
		$('#lat_second').val( latLng.latitude.seconds );
		$('#lng_second').val( latLng.longitude.seconds );
		$('#lat_direction').val( latLng.latitude.direction );
		$('#lng_direction').val( latLng.longitude.direction );

		//$('#lat_time').html( latTime );
	    //$('#lng_time').html( lngTime );
	    
	    viewLocationWindow( location );
	}
	
	return {
		init: function(){
			initMap();
		}
	}
})();